#include "DLClassifier.h"

#define Malloc(type,n) (type *)malloc((n)*sizeof(type))

DLClassifier::DLClassifier( string model_filename )
{
	param.svm_type = C_SVC;
	param.kernel_type = RBF;
	param.degree = 3;
	param.gamma = 0;	// 1/k
	param.coef0 = 0;
	param.nu = 0.5;
	param.cache_size = 100;
	param.C = 100000;
	param.eps = 1e-10;
	param.p = 0.1;
	param.shrinking = 1;
	param.probability = 0;
	param.nr_weight = 0;
	param.weight_label = NULL;
	param.weight = NULL;

	model_file_name = model_filename;
}

vector< vector<double> >  DLClassifier:: batchTesting(vector< vector<double> > data, vector<int> dataclass, vector<int> training)
{
	int nr_class=svm_get_nr_class(model);

	// prepare the testing set
	vector<int> testing;
	vector<int> correctLabel;
	sort( training.begin(), training.end() );

	int idx1 = 0;
	for(int i=0;i< data.size(); i++)
	{
		if( i == training[idx1] )
		{
			idx1++;
			if(idx1 == training.size() )
				break;
		}
		else
		{
			testing.push_back(i);
			correctLabel.push_back(dataclass[i]);
		}
	}

	vector<int> testPredict;
	struct svm_node *x;
	x = (struct svm_node *) malloc( (data[0].size()  + 1)*sizeof(struct svm_node));
	idx1 = 0;
	for(int i=0;i< data.size(); i++)
	{

		if( i == testing[idx1] )
		{

			for( int j = 0; j < data[0].size(); j++)
			{
				x[j].index = j;
				x[j].value = data[i][j];
			}

			x[data[0].size()].index = -1;
			int v = svm_predict(model,x);
			testPredict.push_back(v);

			idx1++;
			if(idx1 == testing.size() )
				break;
		}

	}

	// creating confusion matrix
	for( int i = 0; i < nr_class; i++)
	{
		vector< double > tmp;
		for( int i = 0; i < nr_class; i++)
		{
			tmp.push_back(0);
		}
		confusionMatrix.push_back(tmp);
	}

	for( int i = 0; i < testing.size(); i++)
		confusionMatrix[correctLabel[i] - 1][testPredict[i] - 1] ++;

	for( int i = 0; i < confusionMatrix.size(); i++)
	{	
		int sumval = 0;
		for( int j = 0; j < confusionMatrix.size(); j++)
			sumval+=confusionMatrix[i][j];

		for( int j = 0; j < confusionMatrix.size(); j++)
			confusionMatrix[i][j]=confusionMatrix[i][j]/sumval;		
	}


	return confusionMatrix;
}

int DLClassifier:: batchTraining(vector< vector<double> > data, vector<int> dataclass, vector<int> training)
{
	sort( training.begin(), training.end() );
	prob.l = training.size();
	int elements = training.size() * (data[0].size() + 1 );

	prob.y = Malloc(double,prob.l);
	prob.x = Malloc(struct svm_node *,prob.l);
	x_space = Malloc(struct svm_node,elements);

	int idx = 0;
	int idx1 = 0;
	for(int i=0;i< data.size(); i++)
	{

		if( i == training[idx1] )
		{
			prob.y[idx1] = dataclass[i];
			prob.x[idx1] = &x_space[idx];

			for( int j = 0; j < data[0].size(); j++)
			{
				x_space[idx].index = j;
				x_space[idx].value = data[i][j];
				idx++;
			}

			x_space[idx++].index = -1;

			idx1++;
			if(idx1 == training.size() )
				break;
		}
	}

	if(param.gamma == 0)
	{
		sort(dataclass.begin(), dataclass.end());
		int maxVal = dataclass[dataclass.size()-1];
		param.gamma = 1.0 / maxVal;
	};

	error_msg = svm_check_parameter(&prob,&param);

	if(error_msg)
	{
		cout << error_msg << endl;
		return -1;
	}

	model = svm_train(&prob,&param);	
	svm_save_model(model_file_name.c_str(),model);
}

DLClassifier::~DLClassifier()
{
	svm_destroy_param(&param);
	free(prob.y);
	free(prob.x);
	free(x_space);
	svm_destroy_model(model);
}

int DLClassifier::saveConfusionMatrix(string strFileName)
{
	FILE* f = fopen( strFileName.c_str(), "w+" );

	char pchElement[100];
	for(unsigned int i = 0; i < confusionMatrix.size(); i++) 
	{
		for(unsigned int j = 0; j < confusionMatrix.size()-1; j++) 
		{
			memset( pchElement, 0, 100 );
			sprintf(pchElement, "%4.4lf\t", confusionMatrix[i][j]);
			fwrite( pchElement, 1, strlen(pchElement), f  );
		}

		memset( pchElement, 0, 100 );
		sprintf(pchElement, "%4.4lf\n", confusionMatrix[i][confusionMatrix.size()-1]);
		fwrite( pchElement, 1, strlen(pchElement), f  );
	}
	fclose( f );

	return 1;
}
